package Catalyst::Model;

use strict;
use base qw/Catalyst::Component/;

=head1 NAME

Catalyst::Model - Catalyst Model base class

=head1 SYNOPSIS

See L<Catalyst>.

=head1 DESCRIPTION

Catalyst Model base class.

=head1 AUTHOR

Sebastian Riedel, C<sri@oook.de>

=head1 COPYRIGHT

This program is free software, you can redistribute it and/or modify it under
the same terms as Perl itself.

=cut

1;
